@@ -6,11 +6,16 @@
 
 #ifndef ZCBOR_DEBUG_H__
 #define ZCBOR_DEBUG_H__
+
 #include <stdint.h>
 #include <stdbool.h>
 #include <stddef.h>
 #include "zcbor_common.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 __attribute__((used))
 static void zcbor_print_compare_lines(const uint8_t *str1, const uint8_t *str2, uint32_t size)
 {
@@ -57,4 +62,8 @@ static void zcbor_print_compare_strings_diff(const uint8_t *str1, const uint8_t
 	}
 }
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif /* ZCBOR_DEBUG_H__ */
