@@ -41,6 +41,7 @@ bool zcbor_uint32_decode(zcbor_state_t *state, uint32_t *result);
 bool zcbor_uint64_decode(zcbor_state_t *state, uint64_t *result);
 bool zcbor_size_decode(zcbor_state_t *state, size_t *result);
 bool zcbor_int_decode(zcbor_state_t *state, void *result_int, size_t int_size);
+bool zcbor_uint_decode(zcbor_state_t *state, void *result_uint, size_t uint_size);
 
 /** The following applies to all _expect() functions that don't have docs.
  *
@@ -130,19 +131,31 @@ static inline bool zcbor_tstr_expect_ptr(zcbor_state_t *state, char const *ptr,
 bool zcbor_tag_decode(zcbor_state_t *state, uint32_t *result);
 bool zcbor_tag_expect(zcbor_state_t *state, uint32_t result);
 
-/** Decode and consume a boolean primitive value. */
+/** Decode and consume a simple value. */
+bool zcbor_simple_decode(zcbor_state_t *state, uint8_t *result);
+bool zcbor_simple_expect(zcbor_state_t *state, uint8_t result);
+
+/** Decode and consume a boolean simple value. */
 bool zcbor_bool_decode(zcbor_state_t *state, bool *result);
 bool zcbor_bool_expect(zcbor_state_t *state, bool result);
 
-/** Decode and consume a float */
+/** Decode and consume an IEEE754 float */
+bool zcbor_float16_decode(zcbor_state_t *state, float *result);
+bool zcbor_float16_expect(zcbor_state_t *state, float result);
+bool zcbor_float16_bytes_decode(zcbor_state_t *state, uint16_t *result);
+bool zcbor_float16_bytes_expect(zcbor_state_t *state, uint16_t result);
+bool zcbor_float16_32_decode(zcbor_state_t *state, float *result);
+bool zcbor_float16_32_expect(zcbor_state_t *state, float result);
 bool zcbor_float32_decode(zcbor_state_t *state, float *result);
 bool zcbor_float32_expect(zcbor_state_t *state, float result);
+bool zcbor_float32_64_decode(zcbor_state_t *state, double *result);
+bool zcbor_float32_64_expect(zcbor_state_t *state, double result);
 bool zcbor_float64_decode(zcbor_state_t *state, double *result);
 bool zcbor_float64_expect(zcbor_state_t *state, double result);
 bool zcbor_float_decode(zcbor_state_t *state, double *result);
 bool zcbor_float_expect(zcbor_state_t *state, double result);
 
-/** Consume and expect a "nil"/"undefined" primitive value.
+/** Consume and expect a "nil"/"undefined" simple value.
  *
  * @param[inout] state   The current state of the encoding.
  * @param[in]    unused  Unused parameter to maintain signature parity with
@@ -297,9 +310,9 @@ bool zcbor_list_map_end_force_decode(zcbor_state_t *state);
  * @retval false  If @p decoder failed before having decoded @p min_decode
  *                values.
  */
-bool zcbor_multi_decode(uint_fast32_t min_decode, uint_fast32_t max_decode, uint_fast32_t *num_decode,
+bool zcbor_multi_decode(size_t min_decode, size_t max_decode, size_t *num_decode,
 		zcbor_decoder_t decoder, zcbor_state_t *state, void *result,
-		uint_fast32_t result_len);
+		size_t result_len);
 
 /** Attempt to decode a value that might not be present in the data.
  *
@@ -312,14 +325,14 @@ bool zcbor_multi_decode(uint_fast32_t min_decode, uint_fast32_t max_decode, uint
  *
  * @return Should always return true.
  */
-bool zcbor_present_decode(uint_fast32_t *present,
+bool zcbor_present_decode(bool *present,
 		zcbor_decoder_t decoder,
 		zcbor_state_t *state,
 		void *result);
 
 /** See @ref zcbor_new_state() */
-void zcbor_new_decode_state(zcbor_state_t *state_array, uint_fast32_t n_states,
-		const uint8_t *payload, size_t payload_len, uint_fast32_t elem_count);
+void zcbor_new_decode_state(zcbor_state_t *state_array, size_t n_states,
+		const uint8_t *payload, size_t payload_len, size_t elem_count);
 
 /** Convenience macro for declaring and initializing a state with backups.
  *
