@@ -14,7 +14,7 @@
 
 /** Return value length from additional value.
  */
-static uint_fast32_t additional_len(uint8_t additional)
+static size_t additional_len(uint8_t additional)
 {
 	if (ZCBOR_VALUE_IS_1_BYTE <= additional && additional <= ZCBOR_VALUE_IS_8_BYTES) {
 		/* 24 => 1
@@ -42,6 +42,7 @@ do {\
 	} \
 } while(0)
 
+
 static bool initial_checks(zcbor_state_t *state)
 {
 	ZCBOR_CHECK_ERROR();
@@ -49,6 +50,7 @@ static bool initial_checks(zcbor_state_t *state)
 	return true;
 }
 
+
 static bool type_check(zcbor_state_t *state, zcbor_major_type_t exp_major_type)
 {
 	if (!initial_checks(state)) {
@@ -62,6 +64,7 @@ static bool type_check(zcbor_state_t *state, zcbor_major_type_t exp_major_type)
 	return true;
 }
 
+
 #define INITIAL_CHECKS() \
 do {\
 	if (!initial_checks(state)) { \
@@ -76,11 +79,17 @@ do {\
 	} \
 } while(0)
 
+static void err_restore(zcbor_state_t *state, int err)
+{
+	state->payload = state->payload_bak;
+	state->elem_count++;
+	zcbor_error(state, err);
+}
+
 #define ERR_RESTORE(err) \
 do { \
-	state->payload = state->payload_bak; \
-	state->elem_count++; \
-	ZCBOR_ERR(err); \
+	err_restore(state, err); \
+	ZCBOR_FAIL(); \
 } while(0)
 
 #define FAIL_RESTORE() \
@@ -90,6 +99,7 @@ do { \
 	ZCBOR_FAIL(); \
 } while(0)
 
+
 /** Get a single value.
  *
  * @details @p ppayload must point to the header byte. This function will
@@ -109,7 +119,7 @@ do { \
  *          big to little-endian if necessary (@ref CONFIG_BIG_ENDIAN).
  */
 static bool value_extract(zcbor_state_t *state,
-		void *const result, uint_fast32_t result_len)
+		void *const result, size_t result_len)
 {
 	zcbor_trace();
 	zcbor_assert_state(result_len != 0, "0-length result not supported.\r\n");
@@ -132,7 +142,7 @@ static bool value_extract(zcbor_state_t *state,
 		u8_result[0] = additional;
 #endif /* CONFIG_BIG_ENDIAN */
 	} else {
-		uint_fast32_t len = additional_len(additional);
+		size_t len = additional_len(additional);
 
 		FAIL_AND_DECR_IF(len > result_len, ZCBOR_ERR_INT_SIZE);
 		FAIL_AND_DECR_IF(len == 0, ZCBOR_ERR_ADDITIONAL_INVAL); // additional_len() did not recognize the additional value.
@@ -142,7 +152,7 @@ static bool value_extract(zcbor_state_t *state,
 #ifdef CONFIG_BIG_ENDIAN
 		memcpy(&u8_result[result_len - len], state->payload, len);
 #else
-		for (uint_fast32_t i = 0; i < len; i++) {
+		for (size_t i = 0; i < len; i++) {
 			u8_result[i] = (state->payload)[len - i - 1];
 		}
 #endif /* CONFIG_BIG_ENDIAN */
@@ -183,7 +193,7 @@ bool zcbor_int_decode(zcbor_state_t *state, void *result_int, size_t int_size)
 
 	if (major_type == ZCBOR_MAJOR_TYPE_NINT) {
 		/* Convert from CBOR's representation by flipping all bits. */
-		for (int i = 0; i < int_size; i++) {
+		for (unsigned int i = 0; i < int_size; i++) {
 			result_uint8[i] = (uint8_t)~result_uint8[i];
 		}
 	}
@@ -204,17 +214,24 @@ bool zcbor_int64_decode(zcbor_state_t *state, int64_t *result)
 }
 
 
-bool zcbor_uint32_decode(zcbor_state_t *state, uint32_t *result)
+bool zcbor_uint_decode(zcbor_state_t *state, void *result_uint, size_t uint_size)
 {
 	INITIAL_CHECKS_WITH_TYPE(ZCBOR_MAJOR_TYPE_PINT);
 
-	if (!value_extract(state, result, sizeof(*result))) {
+	if (!value_extract(state, result_uint, uint_size)) {
+		zcbor_print("uint with size %d failed.\r\n", uint_size);
 		ZCBOR_FAIL();
 	}
 	return true;
 }
 
 
+bool zcbor_uint32_decode(zcbor_state_t *state, uint32_t *result)
+{
+	return zcbor_uint_decode(state, result, sizeof(*result));
+}
+
+
 bool zcbor_int32_expect_union(zcbor_state_t *state, int32_t result)
 {
 	if (!zcbor_union_elem_code(state)) {
@@ -275,19 +292,14 @@ bool zcbor_int64_expect(zcbor_state_t *state, int64_t result)
 
 bool zcbor_uint64_decode(zcbor_state_t *state, uint64_t *result)
 {
-	INITIAL_CHECKS_WITH_TYPE(ZCBOR_MAJOR_TYPE_PINT);
-
-	if (!value_extract(state, result, sizeof(*result))) {
-		ZCBOR_FAIL();
-	}
-	return true;
+	return zcbor_uint_decode(state, result, sizeof(*result));
 }
 
 
 #ifdef ZCBOR_SUPPORTS_SIZE_T
 bool zcbor_size_decode(zcbor_state_t *state, size_t *result)
 {
-	return value_extract(state, result, sizeof(size_t));
+	return zcbor_uint_decode(state, result, sizeof(*result));
 }
 #endif
 
@@ -334,15 +346,24 @@ static bool str_start_decode(zcbor_state_t *state,
 	return true;
 }
 
-
-static bool str_overflow_check(zcbor_state_t *state, struct zcbor_string *result)
+static bool str_start_decode_with_overflow_check(zcbor_state_t *state,
+		struct zcbor_string *result, zcbor_major_type_t exp_major_type)
 {
-	if (result->len > (state->payload_end - state->payload)) {
+	bool res = str_start_decode(state, result, exp_major_type);
+
+	if (!res) {
+		ZCBOR_FAIL();
+	}
+
+	/* Casting to size_t is safe since str_start_decode() checks that
+	 * payload_end is bigger that payload. */
+	if (result->len > (size_t)(state->payload_end - state->payload)) {
 		zcbor_print("error: 0x%zu > 0x%zu\r\n",
-		result->len,
-		(state->payload_end - state->payload));
+			result->len,
+			(state->payload_end - state->payload));
 		ERR_RESTORE(ZCBOR_ERR_NO_PAYLOAD);
 	}
+
 	return true;
 }
 
@@ -354,11 +375,7 @@ bool zcbor_bstr_start_decode(zcbor_state_t *state, struct zcbor_string *result)
 		result = &dummy;
 	}
 
-	if(!str_start_decode(state, result, ZCBOR_MAJOR_TYPE_BSTR)) {
-		ZCBOR_FAIL();
-	}
-
-	if (!str_overflow_check(state, result)) {
+	if(!str_start_decode_with_overflow_check(state, result, ZCBOR_MAJOR_TYPE_BSTR)) {
 		ZCBOR_FAIL();
 	}
 
@@ -462,11 +479,7 @@ bool zcbor_is_last_fragment(const struct zcbor_string_fragment *fragment)
 static bool str_decode(zcbor_state_t *state, struct zcbor_string *result,
 		zcbor_major_type_t exp_major_type)
 {
-	if (!str_start_decode(state, result, exp_major_type)) {
-		ZCBOR_FAIL();
-	}
-
-	if (!str_overflow_check(state, result)) {
+	if (!str_start_decode_with_overflow_check(state, result, exp_major_type)) {
 		ZCBOR_FAIL();
 	}
 
@@ -495,8 +508,7 @@ static bool str_expect(zcbor_state_t *state, struct zcbor_string *result,
 	if (!str_decode(state, &tmp_result, exp_major_type)) {
 		ZCBOR_FAIL();
 	}
-	if ((tmp_result.len != result->len)
-			|| memcmp(result->value, tmp_result.value, tmp_result.len)) {
+	if (!zcbor_compare_strings(&tmp_result, result)) {
 		ERR_RESTORE(ZCBOR_ERR_WRONG_VALUE);
 	}
 	return true;
@@ -542,7 +554,7 @@ bool zcbor_tstr_expect(zcbor_state_t *state, struct zcbor_string *result)
 static bool list_map_start_decode(zcbor_state_t *state,
 		zcbor_major_type_t exp_major_type)
 {
-	uint_fast32_t new_elem_count;
+	size_t new_elem_count;
 	bool indefinite_length_array = false;
 
 	INITIAL_CHECKS_WITH_TYPE(exp_major_type);
@@ -603,7 +615,7 @@ static bool array_end_expect(zcbor_state_t *state)
 
 static bool list_map_end_decode(zcbor_state_t *state)
 {
-	uint_fast32_t max_elem_count = 0;
+	size_t max_elem_count = 0;
 
 	if (state->indefinite_length_array) {
 		if (!array_end_expect(state)) {
@@ -646,50 +658,64 @@ bool zcbor_list_map_end_force_decode(zcbor_state_t *state)
 }
 
 
-static bool primx_expect(zcbor_state_t *state, uint8_t result)
+bool zcbor_simple_decode(zcbor_state_t *state, uint8_t *result)
 {
-	uint32_t value;
+	INITIAL_CHECKS_WITH_TYPE(ZCBOR_MAJOR_TYPE_SIMPLE);
 
-	INITIAL_CHECKS_WITH_TYPE(ZCBOR_MAJOR_TYPE_PRIM);
+	/* Simple values must be 0-23 (additional is 0-23) or 24-255 (additional is 24).
+	 * Other additional values are not considered simple values. */
+	ZCBOR_ERR_IF(ADDITIONAL(*state->payload) > 24, ZCBOR_ERR_WRONG_TYPE);
 
-	if (!value_extract(state, &value, sizeof(value))) {
+	if (!value_extract(state, result, sizeof(*result))) {
+		ZCBOR_FAIL();
+	}
+	return true;
+}
+
+
+bool zcbor_simple_expect(zcbor_state_t *state, uint8_t result)
+{
+	uint8_t value;
+
+	if (!zcbor_simple_decode(state, &value)) {
 		ZCBOR_FAIL();
 	}
 
 	if (value != result) {
+		zcbor_print("simple value %u != %u\r\n", value, result);
 		ERR_RESTORE(ZCBOR_ERR_WRONG_VALUE);
 	}
+
 	return true;
 }
 
 
 bool zcbor_nil_expect(zcbor_state_t *state, void *unused)
 {
-	if (!primx_expect(state, 22)) {
-		ZCBOR_FAIL();
-	}
-	return true;
+	(void)unused;
+	return zcbor_simple_expect(state, 22);
 }
 
 
 bool zcbor_undefined_expect(zcbor_state_t *state, void *unused)
 {
-	if (!primx_expect(state, 23)) {
-		ZCBOR_FAIL();
-	}
-	return true;
+	(void)unused;
+	return zcbor_simple_expect(state, 23);
 }
 
 
 bool zcbor_bool_decode(zcbor_state_t *state, bool *result)
 {
-	if (zcbor_bool_expect(state, false)) {
-		*result = false;
-	} else if (zcbor_bool_expect(state, true)) {
-		*result = true;
-	} else {
+	uint8_t value;
+
+	if (!zcbor_simple_decode(state, &value)) {
 		ZCBOR_FAIL();
 	}
+	value -= ZCBOR_BOOL_TO_SIMPLE;
+	if (value > 1) {
+		ERR_RESTORE(ZCBOR_ERR_WRONG_TYPE);
+	}
+	*result = value;
 
 	zcbor_print("boolval: %u\r\n", *result);
 	return true;
@@ -698,17 +724,107 @@ bool zcbor_bool_decode(zcbor_state_t *state, bool *result)
 
 bool zcbor_bool_expect(zcbor_state_t *state, bool result)
 {
-	if (!primx_expect(state, (uint8_t)(!!result) + ZCBOR_BOOL_TO_PRIM)) {
+	return zcbor_simple_expect(state, (uint8_t)(!!result) + ZCBOR_BOOL_TO_SIMPLE);
+}
+
+
+static bool float_check(zcbor_state_t *state, uint8_t additional_val)
+{
+	INITIAL_CHECKS_WITH_TYPE(ZCBOR_MAJOR_TYPE_SIMPLE);
+	ZCBOR_ERR_IF(ADDITIONAL(*state->payload) != additional_val, ZCBOR_ERR_FLOAT_SIZE);
+	return true;
+}
+
+
+bool zcbor_float16_bytes_decode(zcbor_state_t *state, uint16_t *result)
+{
+
+	ZCBOR_FAIL_IF(!float_check(state, ZCBOR_VALUE_IS_2_BYTES));
+
+	if (!value_extract(state, result, sizeof(*result))) {
 		ZCBOR_FAIL();
 	}
+
+	return true;
+}
+
+
+bool zcbor_float16_bytes_expect(zcbor_state_t *state, uint16_t result)
+{
+	uint16_t value;
+
+	if (!zcbor_float16_bytes_decode(state, &value)) {
+		ZCBOR_FAIL();
+	}
+	if (value != result) {
+		ERR_RESTORE(ZCBOR_ERR_WRONG_VALUE);
+	}
+	return true;
+}
+
+
+/* Float16: */
+#define F16_SIGN_OFFS 15 /* Bit offset of the sign bit. */
+#define F16_EXPO_OFFS 10 /* Bit offset of the exponent. */
+#define F16_EXPO_MSK 0x1F /* Bitmask for the exponent (right shifted by F16_EXPO_OFFS). */
+#define F16_MANTISSA_MSK 0x3FF /* Bitmask for the mantissa. */
+#define F16_MIN_EXPO 24 /* Negative exponent of the non-zero float16 value closest to 0 (2^-24) */
+#define F16_MIN (1.0 / (1 << F16_MIN_EXPO)) /* The non-zero float16 value closest to 0 (2^-24) */
+#define F16_BIAS 15 /* The exponent bias of normalized float16 values. */
+
+/* Float32: */
+#define F32_SIGN_OFFS 31 /* Bit offset of the sign bit. */
+#define F32_EXPO_OFFS 23 /* Bit offset of the exponent. */
+#define F32_EXPO_MSK 0xFF /* Bitmask for the exponent (right shifted by F32_EXPO_OFFS). */
+#define F32_BIAS 127 /* The exponent bias of normalized float32 values. */
+
+
+bool zcbor_float16_decode(zcbor_state_t *state, float *result)
+{
+	uint16_t value16;
+
+	if (!zcbor_float16_bytes_decode(state, &value16)) {
+		ZCBOR_FAIL();
+	}
+
+	uint32_t sign = value16 >> F16_SIGN_OFFS;
+	uint32_t expo = (value16 >> F16_EXPO_OFFS) & F16_EXPO_MSK;
+	uint32_t mantissa = value16 & F16_MANTISSA_MSK;
+
+	if ((expo == 0) && (mantissa != 0)) {
+		/* Subnormal float16 - convert to normalized float32 */
+		*result = ((float)mantissa * (float)F16_MIN) * (sign ? -1 : 1);
+	} else {
+		/* Normalized / zero / Infinity / NaN */
+		uint32_t new_expo = (expo == 0 /* zero */) ? 0
+			: (expo == F16_EXPO_MSK /* inf/NaN */) ? F32_EXPO_MSK
+				: (expo + (F32_BIAS - F16_BIAS));
+		uint32_t value32 = (sign << F32_SIGN_OFFS) | (new_expo << F32_EXPO_OFFS)
+			| (mantissa << (F32_EXPO_OFFS - F16_EXPO_OFFS));
+		memcpy(result, &value32, sizeof(*result));
+	}
+
+	return true;
+}
+
+
+bool zcbor_float16_expect(zcbor_state_t *state, float result)
+{
+	float value;
+
+	if (!zcbor_float16_decode(state, &value)) {
+		ZCBOR_FAIL();
+	}
+	if (value != result) {
+		ERR_RESTORE(ZCBOR_ERR_WRONG_VALUE);
+	}
 	return true;
 }
 
 
 bool zcbor_float32_decode(zcbor_state_t *state, float *result)
 {
-	INITIAL_CHECKS_WITH_TYPE(ZCBOR_MAJOR_TYPE_PRIM);
-	ZCBOR_ERR_IF(ADDITIONAL(*state->payload) != ZCBOR_VALUE_IS_4_BYTES, ZCBOR_ERR_FLOAT_SIZE);
+	ZCBOR_FAIL_IF(!float_check(state, ZCBOR_VALUE_IS_4_BYTES));
 
 	if (!value_extract(state, result, sizeof(*result))) {
 		ZCBOR_FAIL();
@@ -732,10 +848,33 @@ bool zcbor_float32_expect(zcbor_state_t *state, float result)
 }
 
 
+bool zcbor_float16_32_decode(zcbor_state_t *state, float *result)
+{
+	if (zcbor_float16_decode(state, result)) {
+		/* Do nothing */
+	} else if (!zcbor_float32_decode(state, result)) {
+		ZCBOR_FAIL();
+	}
+
+	return true;
+}
+
+
+bool zcbor_float16_32_expect(zcbor_state_t *state, float result)
+{
+	if (zcbor_float16_expect(state, (float)result)) {
+		/* Do nothing */
+	} else if (!zcbor_float32_expect(state, result)) {
+		ZCBOR_FAIL();
+	}
+
+	return true;
+}
+
+
 bool zcbor_float64_decode(zcbor_state_t *state, double *result)
 {
-	INITIAL_CHECKS_WITH_TYPE(ZCBOR_MAJOR_TYPE_PRIM);
-	ZCBOR_ERR_IF(ADDITIONAL(*state->payload) != ZCBOR_VALUE_IS_8_BYTES, ZCBOR_ERR_FLOAT_SIZE);
+	ZCBOR_FAIL_IF(!float_check(state, ZCBOR_VALUE_IS_8_BYTES));
 
 	if (!value_extract(state, result, sizeof(*result))) {
 		ZCBOR_FAIL();
@@ -759,7 +898,7 @@ bool zcbor_float64_expect(zcbor_state_t *state, double result)
 }
 
 
-bool zcbor_float_decode(zcbor_state_t *state, double *result)
+bool zcbor_float32_64_decode(zcbor_state_t *state, double *result)
 {
 	float float_result;
 
@@ -773,9 +912,39 @@ bool zcbor_float_decode(zcbor_state_t *state, double *result)
 }
 
 
+bool zcbor_float32_64_expect(zcbor_state_t *state, double result)
+{
+	if (zcbor_float64_expect(state, result)) {
+		/* Do nothing */
+	} else if (!zcbor_float32_expect(state, (float)result)) {
+		ZCBOR_FAIL();
+	}
+
+	return true;
+}
+
+
+bool zcbor_float_decode(zcbor_state_t *state, double *result)
+{
+	float float_result;
+
+	if (zcbor_float16_decode(state, &float_result)) {
+		*result = (double)float_result;
+	} else if (zcbor_float32_decode(state, &float_result)) {
+		*result = (double)float_result;
+	} else if (!zcbor_float64_decode(state, result)) {
+		ZCBOR_FAIL();
+	}
+
+	return true;
+}
+
+
 bool zcbor_float_expect(zcbor_state_t *state, double result)
 {
-	if (zcbor_float32_expect(state, (float)result)) {
+	if (zcbor_float16_expect(state, (float)result)) {
+		/* Do nothing */
+	} else if (zcbor_float32_expect(state, (float)result)) {
 		/* Do nothing */
 	} else if (!zcbor_float64_expect(state, result)) {
 		ZCBOR_FAIL();
@@ -789,50 +958,31 @@ bool zcbor_any_skip(zcbor_state_t *state, void *result)
 {
 	zcbor_assert_state(result == NULL,
 			"'any' type cannot be returned, only skipped.\r\n");
+	(void)result;
 
 	INITIAL_CHECKS();
 	zcbor_major_type_t major_type = MAJOR_TYPE(*state->payload);
 	uint8_t additional = ADDITIONAL(*state->payload);
-	uint_fast32_t value;
-	uint_fast32_t num_decode;
-	uint_fast32_t temp_elem_count;
-	uint_fast32_t elem_count_bak = state->elem_count;
-	uint8_t const *payload_bak = state->payload;
-	uint64_t tag_dummy;
+	uint64_t value = 0; /* In case of indefinite_length_array. */
+	zcbor_state_t state_copy;
 
-	payload_bak = state->payload;
+	memcpy(&state_copy, state, sizeof(zcbor_state_t));
 
-	if (!zcbor_multi_decode(0, ZCBOR_LARGE_ELEM_COUNT, &num_decode,
-			(zcbor_decoder_t *)zcbor_tag_decode, state,
-			(void *)&tag_dummy, 0)) {
-		state->elem_count = elem_count_bak;
-		state->payload = payload_bak;
-		ZCBOR_FAIL();
-	}
+	while (major_type == ZCBOR_MAJOR_TYPE_TAG) {
+		uint32_t tag_dummy;
 
-	if ((major_type == ZCBOR_MAJOR_TYPE_MAP) || (major_type == ZCBOR_MAJOR_TYPE_LIST)) {
-		if (additional == ZCBOR_VALUE_IS_INDEFINITE_LENGTH) {
-			ZCBOR_ERR_IF(state->elem_count == 0, ZCBOR_ERR_LOW_ELEM_COUNT);
-			state->payload++;
-			state->elem_count--;
-			temp_elem_count = state->elem_count;
-			payload_bak = state->payload;
-			state->elem_count = ZCBOR_LARGE_ELEM_COUNT;
-			if (!zcbor_multi_decode(0, ZCBOR_LARGE_ELEM_COUNT, &num_decode,
-					(zcbor_decoder_t *)zcbor_any_skip, state,
-					NULL, 0)
-					|| (state->payload >= state->payload_end)
-					|| !(*(state->payload++) == 0xFF)) {
-				state->elem_count = elem_count_bak;
-				state->payload = payload_bak;
-				ZCBOR_FAIL();
-			}
-			state->elem_count = temp_elem_count;
-			return true;
+		if (!zcbor_tag_decode(&state_copy, &tag_dummy)) {
+			ZCBOR_FAIL();
 		}
+		ZCBOR_ERR_IF(state_copy.payload >= state_copy.payload_end, ZCBOR_ERR_NO_PAYLOAD);
+		major_type = MAJOR_TYPE(*state_copy.payload);
+		additional = ADDITIONAL(*state_copy.payload);
 	}
 
-	if (!value_extract(state, &value, sizeof(value))) {
+	const bool indefinite_length_array = ((additional == ZCBOR_VALUE_IS_INDEFINITE_LENGTH)
+		&& ((major_type == ZCBOR_MAJOR_TYPE_LIST) || (major_type == ZCBOR_MAJOR_TYPE_MAP)));
+
+	if (!indefinite_length_array && !value_extract(&state_copy, &value, sizeof(value))) {
 		/* Can happen because of elem_count (or payload_end) */
 		ZCBOR_FAIL();
 	}
@@ -840,32 +990,42 @@ bool zcbor_any_skip(zcbor_state_t *state, void *result)
 	switch (major_type) {
 		case ZCBOR_MAJOR_TYPE_BSTR:
 		case ZCBOR_MAJOR_TYPE_TSTR:
-			/* 'value' is the length of the BSTR or TSTR */
-			if (value > (state->payload_end - state->payload)) {
-				ZCBOR_ERR(ZCBOR_ERR_NO_PAYLOAD);
-			}
-			(state->payload) += value;
+			/* 'value' is the length of the BSTR or TSTR.
+			 * The cast to size_t is safe because value_extract() above
+			 * checks that payload_end is greater than payload. */
+			ZCBOR_ERR_IF(
+				value > (uint64_t)(state_copy.payload_end - state_copy.payload),
+				ZCBOR_ERR_NO_PAYLOAD);
+			(state_copy.payload) += value;
 			break;
 		case ZCBOR_MAJOR_TYPE_MAP:
-			value *= 2; /* Because all members have a key. */
-			/* Fallthrough */
+			ZCBOR_ERR_IF(value > (SIZE_MAX / 2), ZCBOR_ERR_INT_SIZE);
+			value *= 2;
+			/* fallthrough */
 		case ZCBOR_MAJOR_TYPE_LIST:
-			temp_elem_count = state->elem_count;
-			state->elem_count = value;
-			if (!zcbor_multi_decode(value, value, &num_decode,
-					(zcbor_decoder_t *)zcbor_any_skip, state,
-					NULL, 0)) {
-				state->elem_count = elem_count_bak;
-				state->payload = payload_bak;
+			if (indefinite_length_array) {
+				state_copy.payload++;
+				value = ZCBOR_LARGE_ELEM_COUNT;
+			}
+			state_copy.elem_count = (size_t)value;
+			state_copy.indefinite_length_array = indefinite_length_array;
+			while (!zcbor_array_at_end(&state_copy)) {
+				if (!zcbor_any_skip(&state_copy, NULL)) {
+					ZCBOR_FAIL();
+				}
+			}
+			if (indefinite_length_array && !array_end_expect(&state_copy)) {
 				ZCBOR_FAIL();
 			}
-			state->elem_count = temp_elem_count;
 			break;
 		default:
 			/* Do nothing */
 			break;
 	}
 
+	state->payload = state_copy.payload;
+	state->elem_count--;
+
 	return true;
 }
 
@@ -896,18 +1056,18 @@ bool zcbor_tag_expect(zcbor_state_t *state, uint32_t result)
 }
 
 
-bool zcbor_multi_decode(uint_fast32_t min_decode,
-		uint_fast32_t max_decode,
-		uint_fast32_t *num_decode,
+bool zcbor_multi_decode(size_t min_decode,
+		size_t max_decode,
+		size_t *num_decode,
 		zcbor_decoder_t decoder,
 		zcbor_state_t *state,
 		void *result,
-		uint_fast32_t result_len)
+		size_t result_len)
 {
 	ZCBOR_CHECK_ERROR();
-	for (uint_fast32_t i = 0; i < max_decode; i++) {
+	for (size_t i = 0; i < max_decode; i++) {
 		uint8_t const *payload_bak = state->payload;
-		uint_fast32_t elem_count_bak = state->elem_count;
+		size_t elem_count_bak = state->elem_count;
 
 		if (!decoder(state,
 				(uint8_t *)result + i*result_len)) {
@@ -925,23 +1085,23 @@ bool zcbor_multi_decode(uint_fast32_t min_decode,
 }
 
 
-bool zcbor_present_decode(uint_fast32_t *present,
+bool zcbor_present_decode(bool *present,
 		zcbor_decoder_t decoder,
 		zcbor_state_t *state,
 		void *result)
 {
-	uint_fast32_t num_decode;
+	size_t num_decode;
 	bool retval = zcbor_multi_decode(0, 1, &num_decode, decoder, state, result, 0);
 
 	zcbor_assert_state(retval, "zcbor_multi_decode should not fail with these parameters.\r\n");
 
-	*present = num_decode;
+	*present = !!num_decode;
 	return retval;
 }
 
 
-void zcbor_new_decode_state(zcbor_state_t *state_array, uint_fast32_t n_states,
-		const uint8_t *payload, size_t payload_len, uint_fast32_t elem_count)
+void zcbor_new_decode_state(zcbor_state_t *state_array, size_t n_states,
+		const uint8_t *payload, size_t payload_len, size_t elem_count)
 {
 	zcbor_new_state(state_array, n_states, payload, payload_len, elem_count);
 }
